/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.ExpressionException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Collections;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.genclass.RdbFieldDesc;
import org.objectweb.jorm.mapper.rdb.genclass.RdbGenClassProp;


import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.Iterator;

public class Olstore_userMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String address_NAME = "address";
	protected static final String emailAdd_NAME = "emailAdd";
	protected static final String fname_NAME = "fname";
	protected static final String friends_NAME = "friends";
	protected static final String jonasCMRFriend_User_NAME = "jonasCMRFriend_User";
	protected static final String lname_NAME = "lname";
	protected static final String orders_NAME = "orders";
	protected static final String passwd_NAME = "passwd";
	protected static final String phoneNum_NAME = "phoneNum";
	protected static final String purchased_NAME = "purchased";
	protected static final String role_NAME = "role";
	protected static final String username_NAME = "username";
	//Hidden fields
	protected static final String address_JONASAUTOPKFIELD_NAME = "address_JONASAUTOPKFIELD";

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.Olstore_user";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder addressPnc;
	public PNameCoder friendsPnc;
	PClassMapping friendsGCM;
	public PNameCoder jonasCMRFriend_UserPnc;
	PClassMapping jonasCMRFriend_UserGCM;
	public PNameCoder ordersPnc;
	PClassMapping ordersGCM;
	public PNameCoder purchasedPnc;
	PClassMapping purchasedGCM;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public Olstore_userMapping() {
 	}
	public PNameCoder getAddressPNameCoder() {
	    return addressPnc;
	}
	public PNameCoder getFriendsPNameCoder() {
	    return friendsPnc;
	}
	public PNameCoder getJonasCMRFriend_UserPNameCoder() {
	    return jonasCMRFriend_UserPnc;
	}
	public PNameCoder getOrdersPNameCoder() {
	    return ordersPnc;
	}
	public PNameCoder getPurchasedPNameCoder() {
	    return purchasedPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (address_NAME.equals(fn)) {
			addressPnc = (PNamingContext) pnc;
			return;
		}
		if (friends_NAME.equals(fn)) {
			friendsPnc = (PNamingContext) pnc;
			return;
		}
		if (jonasCMRFriend_User_NAME.equals(fn)) {
			jonasCMRFriend_UserPnc = (PNamingContext) pnc;
			return;
		}
		if (orders_NAME.equals(fn)) {
			ordersPnc = (PNamingContext) pnc;
			return;
		}
		if (purchased_NAME.equals(fn)) {
			purchasedPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		if (friends_NAME.equals(fn)) {
			friendsGCM = pcm;
			return;
		}
		if (jonasCMRFriend_User_NAME.equals(fn)) {
			jonasCMRFriend_UserGCM = pcm;
			return;
		}
		if (orders_NAME.equals(fn)) {
			ordersGCM = pcm;
			return;
		}
		if (purchased_NAME.equals(fn)) {
			purchasedGCM = pcm;
			return;
		}
		throw new UnsupportedOperationException("This class does not contain a " + fn + " field referencing a generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		PType ptype = mapper.getPTypeSpace().createPType(
			"jorm.Olstore_user",
			SUPER_TYPES);
		setPType(ptype);
        // PType Assignement to the PNC managing the reference address
        String[][] addressICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.Olstore_address",
            addressICDef);
        addressPnc.setPType(ptype);
        
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}

         // Initialization of the gen class /jorm.Olstore_user/jonasCMRFriend_User
        FieldDesc[] fds = null;
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "null", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) jonasCMRFriend_UserGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) jonasCMRFriend_UserGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_friend_id", "null", PTypeSpace.OBJINT, "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) jonasCMRFriend_UserGCM).setElemDesc(fds);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setTableName("olstore_friend");
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setColocated(true);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setColocatedMaster(false);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setReadOnly(true);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setTypeConverter(adapter);
        String[] jonasCMRFriend_UserGCDef = {"java.util.Collection"};
        String[][] jonasCMRFriend_UserICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            jonasCMRFriend_UserGCDef,
            "jorm.Olstore_friend",
            jonasCMRFriend_UserICDef);
        ((GenClassProp) jonasCMRFriend_UserGCM).setPType(ptype);
        jonasCMRFriend_UserPnc.setPType(ptype);
        jonasCMRFriend_UserGCM.init(m, null);

         // Initialization of the gen class /jorm.Olstore_user/purchased
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "null", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) purchasedGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) purchasedGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_item_itemId", "null", PTypeSpace.OBJINT, "olstore_item_itemId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) purchasedGCM).setElemDesc(fds);
			((RdbGenClassProp) purchasedGCM).setTableName("OLSTORE_USER_OLSTORE_ITEM");
			((RdbGenClassProp) purchasedGCM).setColocated(false);
			((RdbGenClassProp) purchasedGCM).setColocatedMaster(false);
			((RdbGenClassProp) purchasedGCM).setReadOnly(true);
			((RdbGenClassProp) purchasedGCM).setTypeConverter(adapter);
        String[] purchasedGCDef = {"java.util.Collection"};
        String[][] purchasedICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            purchasedGCDef,
            "jorm.Olstore_item",
            purchasedICDef);
        ((GenClassProp) purchasedGCM).setPType(ptype);
        purchasedPnc.setPType(ptype);
        purchasedGCM.init(m, null);

         // Initialization of the gen class /jorm.Olstore_user/orders
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "null", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) ordersGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) ordersGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_order_orderId", "null", PTypeSpace.OBJINT, "orderId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) ordersGCM).setElemDesc(fds);
			((RdbGenClassProp) ordersGCM).setTableName("olstore_order");
			((RdbGenClassProp) ordersGCM).setColocated(true);
			((RdbGenClassProp) ordersGCM).setColocatedMaster(false);
			((RdbGenClassProp) ordersGCM).setReadOnly(true);
			((RdbGenClassProp) ordersGCM).setTypeConverter(adapter);
        String[] ordersGCDef = {"java.util.Collection"};
        String[][] ordersICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            ordersGCDef,
            "jorm.Olstore_order",
            ordersICDef);
        ((GenClassProp) ordersGCM).setPType(ptype);
        ordersPnc.setPType(ptype);
        ordersGCM.init(m, null);

         // Initialization of the gen class /jorm.Olstore_user/friends
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "null", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) friendsGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) friendsGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_friend_id", "null", PTypeSpace.OBJINT, "olstore_friend_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) friendsGCM).setElemDesc(fds);
			((RdbGenClassProp) friendsGCM).setTableName("OLSTORE_USER_OLSTORE_FRIEND");
			((RdbGenClassProp) friendsGCM).setColocated(false);
			((RdbGenClassProp) friendsGCM).setColocatedMaster(false);
			((RdbGenClassProp) friendsGCM).setReadOnly(true);
			((RdbGenClassProp) friendsGCM).setTypeConverter(adapter);
        String[] friendsGCDef = {"java.util.Collection"};
        String[][] friendsICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            friendsGCDef,
            "jorm.Olstore_friend",
            friendsICDef);
        ((GenClassProp) friendsGCM).setPType(ptype);
        friendsPnc.setPType(ptype);
        friendsGCM.init(m, null);
 		mapper.getPMappingStructuresManager().declareClass(JORM_CLASS_NAME);
        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager)
            mapper.getPMappingStructuresManager();
	         // Initialization of the structures of the class jorm.Olstore_user
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "lname", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "fname", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "emailAdd", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "passwd", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, -1, -1), true, true, true);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "role", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "phoneNum", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "olstore_address_JPK_", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, -1, -1), false, false, true);
         // Initialization of the structures of the gen class /jorm.Olstore_user/jonasCMRFriend_User
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_friend", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_friend", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
         // Initialization of the structures of the gen class /jorm.Olstore_user/purchased
        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_ITEM", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_ITEM", "olstore_item_itemId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
         // Initialization of the structures of the gen class /jorm.Olstore_user/orders
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_order", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_order", "orderId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
         // Initialization of the structures of the gen class /jorm.Olstore_user/friends
        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_FRIEND", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_FRIEND", "olstore_friend_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		} catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 

		computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		//dependencies
        mapper.getPMappingStructuresManager().addDependency(JORM_CLASS_NAME, "jorm.Olstore_friend");
        mapper.getPMappingStructuresManager().addDependency(JORM_CLASS_NAME, "jorm.Olstore_order");
        mapper.getPMappingStructuresManager().addDependency(JORM_CLASS_NAME, "jorm.Olstore_address");
        mapper.getPMappingStructuresManager().addDependency(JORM_CLASS_NAME, "jorm.Olstore_item");
		classDefined();
	}

	protected void classDefined() throws PException {
		//register class in the cluster
        mapper.getPMappingStructuresManager().classDefined(JORM_CLASS_NAME);
	}

	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (address_NAME.equals(fn))
				return addressPnc;
			if (friends_NAME.equals(fn))
				return friendsPnc;
			if (jonasCMRFriend_User_NAME.equals(fn))
				return jonasCMRFriend_UserPnc;
			if (orders_NAME.equals(fn))
				return ordersPnc;
			if (purchased_NAME.equals(fn))
				return purchasedPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			if (friends_NAME.equals(fn))
				return friendsGCM;
			if (jonasCMRFriend_User_NAME.equals(fn))
				return jonasCMRFriend_UserGCM;
			if (orders_NAME.equals(fn))
				return ordersGCM;
			if (purchased_NAME.equals(fn))
				return purchasedGCM;
			throw new UnsupportedOperationException("This class does not contain a " + fn + " field which would reference a generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new Olstore_userBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			PreparedStatement ps = connection.prepareStatement(
				(prefetching ? prefetchedExtentQuery : extentQuery));
            it = new Olstore_userResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, logger, 8 + 1, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <Olstore_user>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <Olstore_user>.");
        }
		if (!withSubType) {
			return it;
		}
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getDirectSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return the current iterator
			return it;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			PMapper mapper = getPMapper();
			subPCMs = new PClassMapping[subTypes.length];
			for(int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return  (it.hasNext()
			? new MultiplePNameIterator(subPCMs, it, conn, prefetching, txctx)
			: new MultiplePNameIterator(subPCMs, conn, prefetching, txctx));
	}

    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
        PClassMapping gcm = null;
		addressPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, address_NAME, "jorm.Olstore_address");
        friendsPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, friends_NAME, new String[]{"java.util.Collection"});
        friendsGCM = rc.getGenClassMapping(JORM_CLASS_NAME, friends_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_friend");
        gcm = friendsGCM;
        jonasCMRFriend_UserPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, jonasCMRFriend_User_NAME, new String[]{"java.util.Collection"});
        jonasCMRFriend_UserGCM = rc.getGenClassMapping(JORM_CLASS_NAME, jonasCMRFriend_User_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_friend");
        gcm = jonasCMRFriend_UserGCM;
        ordersPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, orders_NAME, new String[]{"java.util.Collection"});
        ordersGCM = rc.getGenClassMapping(JORM_CLASS_NAME, orders_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_order");
        gcm = ordersGCM;
        purchasedPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, purchased_NAME, new String[]{"java.util.Collection"});
        purchasedGCM = rc.getGenClassMapping(JORM_CLASS_NAME, purchased_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_item");
        gcm = purchasedGCM;
    }

	public PType getPType() {
		return binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}




	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
		((Loggable) friendsGCM).setLogger(l);
		((Loggable) jonasCMRFriend_UserGCM).setLogger(l);
		((Loggable) ordersGCM).setLogger(l);
		((Loggable) purchasedGCM).setLogger(l);
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("olstore_user");
		JoinedTable.Join jtj = null;
        return jt;
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("olstore_user.emailAdd, olstore_user.fname, olstore_user.lname, olstore_user.olstore_address_JPK_, olstore_user.passwd, olstore_user.phoneNum, olstore_user.role, olstore_user.username",
			Collections.singletonList(getJoinedTable()),
			"olstore_user.username = ?",
			0,
			Integer.MAX_VALUE);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		extentQuery = adapter.getQuery("olstore_user.username",
			Collections.singletonList(getJoinedTable()),
			null,
			0,
			Integer.MAX_VALUE);
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		prefetchedExtentQuery = adapter.getQuery("olstore_user.emailAdd, olstore_user.fname, olstore_user.lname, olstore_user.olstore_address_JPK_, olstore_user.passwd, olstore_user.phoneNum, olstore_user.role, olstore_user.username, olstore_user.username",
			Collections.singletonList(getJoinedTable()),
			null,
			0,
			Integer.MAX_VALUE);

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return binder.decodeString(adapter.getString(rs, startindex, null));
	}
 

}


class Olstore_userResultIterator implements PNameIterator {
    ResultSet rs;
	private PBinder binder;
	private Olstore_userMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	Olstore_userResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		Logger l,
		int pi,
		Olstore_userMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (prefetching) {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, this);
        } else {
        	pnameIndex = 1;
        }
        hasNext = (prefetching) ? tc.next() : rs.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (prefetching) ? tc.next() : rs.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (prefetching) ? tc.next() : rs.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, pnameIndex, null);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }
}












